#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ElbiaAdmin Python-lib version 2.1.8
# Copyright (c) 2010-2022, Jozef Sudolsky, ELBIA, s. r. o. (jozef@sudolsky.sk)
# All rights reserved.
# BSD Licence
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     1. Redistributions of source code must retain the above copyright notice,
#        this list of conditions and the following disclaimer.
#
#     2. Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#
#     3. Neither the name of ElbiaAdmin Python-lib nor the names of its
#        contributors may be used to endorse or promote products derived from
#        this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
################################################################################
import socket
import urllib2
import logging

import suds

# oprava cache - uplne vypnutie
class my_suds_client(suds.client.Client):
	def __init__(self, url, **kwargs):
		"""
		@param url: The URL for the WSDL.
		@type url: str
		@param kwargs: keyword arguments.
		@see: L{Options}
		"""
		from suds.options import Options
		from suds.transport.https import HttpAuthenticated
		from suds.reader import DefinitionsReader
		from suds.wsdl import Definitions
		from suds.plugin import PluginContainer
		from suds.client import Factory
		from suds.client import ServiceSelector
		from suds.servicedefinition import ServiceDefinition

		options = Options()
		options.transport = HttpAuthenticated()
		self.options = options
		#options.cache = ObjectCache(days=1)
		self.set_options(**kwargs)
		reader = DefinitionsReader(options, Definitions)
		self.wsdl = reader.open(url)
		plugins = PluginContainer(options.plugins)
		plugins.init.initialized(wsdl=self.wsdl)
		self.factory = Factory(self.wsdl)
		self.service = ServiceSelector(self, self.wsdl.services)
		self.sd = []
		for s in self.wsdl.services:
			sd = ServiceDefinition(self.wsdl, s)
			self.sd.append(sd)
		self.messages = dict(tx=None, rx=None)

class ElbiaAdmin:

	EXPIRED_STATE = "expired"
	BLOCKED_STATE = "blocked"
	LOGIN_OK_STATE = "ok"
	CHPASS_OK_STATE = "ok"
	TRAN_OK_STATE = "ok"
	DIRECTOR_WSDL = "ws.elbiaadmin.sk/?wsdl"

	def __init__(self, login=None, password=None, debug=False, relogin=True, cache=None):
		self.__login__ = login
		self.__password__ = password
		self.debug = debug
		self.__relogin_on_expire__ = relogin
		self.cache = cache
		self.__token__ = None
		self.__web_backend_wsdl__ = None
		self.__mail_backend_wsdl__ = None
		self.__dns_backend_wsdl__ = None
		self.__nic_backend_wsdl__ = None
		self.__sms_backend_wsdl__ = None
		self.__invoicing_backend_wsdl__ = None
		self.clients = {}

	def _get_client(self, btype):
		if btype not in self.clients:
			if btype == "DI":
				wsdl = self.DIRECTOR_WSDL
			elif btype == "W":
				if not self.__web_backend_wsdl__:
					return False
				wsdl = self.__web_backend_wsdl__
			elif btype == "M":
				if not self.__mail_backend_wsdl__:
					return False
				wsdl = self.__mail_backend_wsdl__
			elif btype == "D":
				if not self.__dns_backend_wsdl__:
					return False
				wsdl = self.__dns_backend_wsdl__
			elif btype == "N":
				if not self.__nic_backend_wsdl__:
					return False
				wsdl = self.__nic_backend_wsdl__
			elif btype == "S":
				if not self.__sms_backend_wsdl__:
					return False
				wsdl = self.__sms_backend_wsdl__
			elif btype == "I":
				if not self.__invoicing_backend_wsdl__:
					return False
				wsdl = self.__invoicing_backend_wsdl__
			sudsImportDoctor = None
			if btype == "I":
				imp = suds.xsd.doctor.Import("http://schemas.xmlsoap.org/soap/encoding/")
				sudsImportDoctor = suds.xsd.doctor.ImportDoctor(imp)
			self.clients[btype] = my_suds_client("https://%s" % wsdl, cache=self.cache, doctor=sudsImportDoctor)
		return self.clients[btype]

	def _convert_result(self, result):
		if isinstance(result, suds.sudsobject.Object):
			result = dict(suds.sudsobject.items(result))
			# list
			if len(result) == 1 and isinstance(result[result.keys()[0]], list):
				result = result[result.keys()[0]]
				for i in xrange(len(result)):
					result[i] = self._convert_result(result[i])
			# dict
			else:
				for i in result:
					result[i] = self._convert_result(result[i])
		elif isinstance(result, list):
			for i in xrange(len(result)):
				result[i] = self._convert_result(result[i])
		else:
			if isinstance(result, suds.sax.text.Text):
				result = unicode(result)
		return result

	def _do_command(self, command, args, btype, login=True, add_token=True):
		while True:
			if login and not self.__token__:
				result = self._do_login()
				# ak prihlasenie nebolo uspesne, nema zmysel pokracovat
				if result["state"] != self.LOGIN_OK_STATE:
					return result
				# kedze sme neboli lognuty, token sa tam nepridal
				if add_token:
					args = (self.__token__,) + args[1:]
			if self.debug:
				logging.basicConfig(level=logging.INFO)
				logging.getLogger("suds.client").setLevel(logging.DEBUG)
			args2 = []
			for i in args:
				if isinstance(i, list):
					# prazdne pole
					if len(i) == 0:
						args2.append(None)
					else:
						if len(i) == 1:
							i = (i[0],)
						# pole stringov
						if isinstance(i[0], basestring):
							args2.append({"string": i})
						# pole integerov
						elif isinstance(i[0], int):
							args2.append({"integer": i})
						# pole niecoho ineho
						else:
							args2.append(i)
				else:
					args2.append(i)
			try:
				client = self._get_client(btype)
				if not client:
					return {"state": "noperm"}
				caller = getattr(client.service, command)
				result = caller(*args2)
			except urllib2.URLError:
				return {"state": "conn_err"}
			result = self._convert_result(result)
			if result["state"] == self.EXPIRED_STATE:
				# vynulujem nastavenia, kedze su uz neplatne
				self.__token__ = None
				self.__web_backend_wsdl__ = None
				self.__mail_backend_wsdl__ = None
				self.__dns_backend_wsdl__ = None
				self.__nic_backend_wsdl__ = None
				self.__sms_backend_wsdl__ = None
				self.__invoicing_backend_wsdl__ = None
				# ak sa nema robit re-login, returnem
				if not self.__relogin_on_expire__:
					return result
			else:
				return result

	def _do_login(self):
		result = self._do_command("do_login", (self.__login__, self.__password__), "DI", False)
		if result["state"] == self.LOGIN_OK_STATE:
			self.__token__ = result["data"]["token"]
			self.__permissions__ = result["data"]["permissions"]
			self.__services__ = result["data"]["services"]
			# TODO: urobit univerzalnejsie
			for i in self.__services__:
				if i["type"] == "W":
					self.__web_backend_wsdl__ = i["backends"]["W"]["api_wsdl"]
					self.__mail_backend_wsdl__ = i["backends"]["M"]["api_wsdl"]
					self.__dns_backend_wsdl__ = i["backends"]["D"]["api_wsdl"]
				elif i["type"] == "N":
					self.__nic_backend_wsdl__ = i["backends"]["N"]["api_wsdl"]
				elif i["type"] == "S":
					self.__sms_backend_wsdl__ = i["backends"]["S"]["api_wsdl"]
				elif i["type"] == "I":
					self.__invoicing_backend_wsdl__ = i["backends"]["I"]["api_wsdl"]
		return result

	def _do_logout(self):
		if self.__token__:
			result = self._do_command("do_logout", (self.__token__,), "DI", False)
			self.__token__ = None
			self.__web_backend_wsdl__ = None
			self.__mail_backend_wsdl__ = None
			self.__dns_backend_wsdl__ = None
			self.__nic_backend_wsdl__ = None
			self.__sms_backend_wsdl__ = None
			self.__invoicing_backend_wsdl__ = None
			return result
		return False

	def set_login(self, login, password):
		if login != None:
			self.__login__ = login
		if password != None:
			self.__password__ = password

	def change_password(self, pass_old, pass_new):
		result = self._do_command("change_password", (self.__token__, pass_old, pass_new), "DI")
		# ak zmena hesla bola uspesna, aktualizujeme heslo aj v pamati
		if result["state"] == self.CHPASS_OK_STATE:
			self.set_login(self, None, pass_new)
		return result

	def get_user_info(self):
		return self._do_command("get_user_info", (self.__token__,), "DI")

	def send_message(self, message, service_type):
		return self._do_command("send_message", (self.__token__, message, service_type), "DI")

	def get_cart_items(self):
		return self._do_command("get_cart_items", (self.__token__,), "DI")

	def remove_cart_item(self, itm_id):
		return self._do_command("remove_cart_item", (self.__token__, itm_id), "DI")

	def create_order_from_cart(self):
		return self._do_command("create_order_from_cart", (self.__token__,), "DI")

	def order_credit(self, count):
		return self._do_command("order_credit", (self.__token__, count), "DI")

	def get_credit(self):
		return self._do_command("get_credit", (self.__token__,), "DI")

	def get_credit_movements(self, poffset, pcount, fservice_type=None, flabel=None, fdateaddfrom=None, fdateaddto=None):
		return self._do_command("get_credit_movements", (self.__token__, poffset, pcount, fservice_type, flabel, fdateaddfrom, fdateaddto), "DI")

	def __del__(self):
		self._do_logout()
